﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Drawing.Design;

namespace Infragistics.Controls
{
    /// <summary>
    /// 
    /// </summary>
    public class ReadOnlyPropertyDescriptor : PropertyDescriptor
    {
        private PropertyDescriptor desc;

        /// <summary>
        /// Initializes a new instance of the <see cref="ReadOnlyPropertyDescriptor"/> class.
        /// </summary>
        /// <param name="desc">The desc.</param>
        public ReadOnlyPropertyDescriptor(PropertyDescriptor desc)
            : base(desc.Name, null)
        {
            this.desc = desc;
        }

        /// <summary>
        /// Gets the collection of attributes for this member.
        /// </summary>
        /// <value></value>
        public override AttributeCollection Attributes
        {
            get
            {
                return this.desc.Attributes;
            }
        }

        /// <summary>
        /// Returns whether resetting an object changes its value.
        /// </summary>
        /// <param name="component">The component to test for reset capability.</param>
        /// <returns>
        /// 	<see langword="true "/> if
        /// resetting the component changes its value;
        /// otherwise, <see langword="false"/>.
        /// </returns>
        public override bool CanResetValue(object component)
        {
            return true;
        }

        /// <summary>
        /// Gets the type of the component this property is bound to.
        /// </summary>
        /// <value></value>
        public override Type ComponentType
        {
            get
            {
                return this.desc.ComponentType;
            }
        }

        /// <summary>
        /// Gets the name that can be displayed in a window, such as a
        /// Properties window.
        /// </summary>
        /// <value></value>
        public override string DisplayName
        {
            get
            {
                return this.desc.DisplayName;
            }
        }

        /// <summary>
        /// Gets the current value of the property on a component.
        /// </summary>
        /// <param name="component">The component with the property for which to retrieve the value.</param>
        /// <returns>
        /// The value of a property for a given component.
        /// </returns>
        public override object GetValue(object component)
        {
            object obj = this.desc.GetValue(component);
            object readOnlyObj = new ReadOnlyObject(obj);

            return readOnlyObj;
        }

        /// <summary>
        /// Gets a value
        /// indicating whether this property is read-only.
        /// </summary>
        /// <value></value>
        public override bool IsReadOnly
        {
            get { return true; }
        }

        /// <summary>
        /// Gets the name of the member.
        /// </summary>
        /// <value></value>
        public override string Name
        {
            get { return this.desc.Name; }
        }

        /// <summary>
        /// Gets the type of the property.
        /// </summary>
        /// <value></value>
        public override Type PropertyType
        {
            get { return this.desc.PropertyType; }
        }

        /// <summary>
        /// Resets the value for this property of the component
        /// to the default value.
        /// </summary>
        /// <param name="component">The component with the property value that is to be reset to the default value.</param>
        public override void ResetValue(object component)
        {
        }

        /// <summary>
        /// Determines a value indicating whether the
        /// value of this property needs to be persisted.
        /// </summary>
        /// <param name="component">The component with the property to be examined for persistence.</param>
        /// <returns>
        /// 	<see langword="true "/>if the property should be persisted;
        /// otherwise, <see langword="false"/>.
        /// </returns>
        public override bool ShouldSerializeValue(object component)
        {
            return false;
        }

        /// <summary>
        /// Sets the value of the component to a different value.
        /// </summary>
        /// <param name="component">The component with the property value that is to be set.</param>
        /// <param name="value">The new value.</param>
        public override void SetValue(object component, object value)
        {
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="editorBaseType"></param>
        /// <returns></returns>
        public override object GetEditor(Type editorBaseType)
        {
            return new UITypeEditor();
        }
    }
}
